import soundfile as sf
import numpy as np
import os
import subprocess

# Definiere den Pfad zu ffmpeg
ffmpeg_path = "C:\\ffmpeg\\bin\\ffmpeg.exe"

# Definiere ein Array von Frequenzen für die linearen Frequenzänderungen
frequencies = [10.5, 11.5, 12.0, 13.0, 14.0]  # Array von Frequenzen in Hz

#4 Hz Ruhe und Phantasie
#5 Hz Kehlkopf, Lunge Herz
#6 Hz Arme
#7 Hz Handgelenke
#8 Hz Finger, Füße, Gehör
#10 Hz Serotonin Release
#11 Hz Dopamin Release
#14 Hz sekundärer visueller Kortex
#15 Hz primärer visueller Kortex
#16 Hz HPA-Achse, Pobacken
#17 Hz Leber, Magen
#18 Hz Gedächtnis, Konzentration
#19 Hz Long-COVID, Geschmack, Geruch, Konzentration
#20 Hz Lunge, Niere


# Erstelle den String "pre" aus dem Array "frequencies", gerundet und mit einem Minuszeichen getrennt
pre = "-".join([str(round(freq)) for freq in frequencies]) + "-Hz"
print(pre)

def split_channels(input_file, output_file_left, output_file_right, output_file_combined):
    try:
        # Laden der Audiodatei
        data, samplerate = sf.read(input_file, always_2d=True)

        # Überprüfen, ob die Datei zwei Kanäle hat (Stereo)
        if data.shape[1] != 2:
            print(f"Fehler: Die Audiodatei {input_file} muss stereo sein (zwei Kanäle).")
            return

        # Extrahiere den linken und rechten Kanal
        left_channel = data[:, 0]
        right_channel = data[:, 1]

        # Frequenzänderung des rechten Kanals anhand des Frequenzarrays
        t = np.arange(len(right_channel)) / samplerate
        freq_change = np.zeros(len(t))

        # Teile das Signal gleichmäßig für jede Frequenz im Array auf
        segment_length = len(t) // len(frequencies)
        for i, freq in enumerate(frequencies):
            start_idx = i * segment_length
            end_idx = (i + 1) * segment_length if i < len(frequencies) - 1 else len(t)
            freq_change[start_idx:end_idx] = freq

        right_channel = right_channel * np.cos(2 * np.pi * freq_change * t)

        # Falls die Dateien bereits existieren, lösche sie, um sie zu überschreiben
        if os.path.exists(output_file_left):
            os.remove(output_file_left)
        if os.path.exists(output_file_right):
            os.remove(output_file_right)
        if os.path.exists(output_file_combined):
            os.remove(output_file_combined)

        # Speichern des linken Kanals
        sf.write(output_file_left, left_channel, samplerate)
        print(f"Die Datei wurde erfolgreich als {output_file_left} gespeichert.")

        # Speichern des rechten Kanals mit geänderter Frequenz
        sf.write(output_file_right, right_channel, samplerate)
        print(f"Die Datei wurde erfolgreich als {output_file_right} gespeichert.")

        # Zusammenfügen der beiden Kanäle zu einer Stereo-Datei
        ffmpeg_command = [
            ffmpeg_path,
            "-i", output_file_left,
            "-i", output_file_right,
            "-filter_complex", "[0:a][1:a]join=inputs=2:channel_layout=stereo[a]",
            "-map", "[a]",
            output_file_combined
        ]

        print(f"Starte das Zusammenfügen der Kanäle für {output_file_combined}...")
        result = subprocess.run(ffmpeg_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        if result.returncode != 0:
            print(f"Fehler beim Zusammenfügen der Kanäle: {result.stderr}")
        else:
            print(f"Die Datei wurde erfolgreich als {output_file_combined} gespeichert.")

        # Lösche die temporären Dateien für linken und rechten Kanal
        if os.path.exists(output_file_left):
            os.remove(output_file_left)
        if os.path.exists(output_file_right):
            os.remove(output_file_right)

    except Exception as e:
        print(f"Ein unerwarteter Fehler ist aufgetreten: {e}")

if __name__ == "__main__":
    input_folder = "input"
    output_folder = "output"
    output_prefix = f"binaural_{pre}_"

    # Prüfe, ob die Ordner input und output existieren und erstelle sie gegebenenfalls
    if not os.path.exists(input_folder):
        os.makedirs(input_folder)
        print(f"Der Ordner {input_folder} wurde erstellt.")
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)
        print(f"Der Ordner {output_folder} wurde erstellt.")
    else:
        # Stelle sicher, dass der Ausgabeordner existiert
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)

        for filename in os.listdir(input_folder):
            if filename.endswith(".mp3"):
                input_file = os.path.join(input_folder, filename)
                base_filename = os.path.splitext(filename)[0]
                output_file_left = os.path.join(output_folder, f"{output_prefix}{base_filename}_left.mp3")
                output_file_right = os.path.join(output_folder, f"{output_prefix}{base_filename}_right.mp3")
                output_file_combined = os.path.join(output_folder, f"{output_prefix}{base_filename}.mp3")
                split_channels(input_file, output_file_left, output_file_right, output_file_combined)
